/* netguard-2.0.0/netguard.c */
/* Copyright (c) 2000 monk <lizard8@iname.com>. */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifdef ETHERTAP
 #error ETHERTAP does not work right now. Sorry.
#endif


#ifdef PCAP
 #ifdef ETHERTAP
  #error Please, define PCAP _or_ ETHERTAP! Not both.
 #endif
#endif

#ifndef PCAP
 #ifndef ETHERTAP
  #error No capture method defined!
 #endif
#endif 

#define SAFE_UID	99
#define SAFE_GID	98

#include "version.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>
#ifdef PCAP
#include <pcap.h>
#endif
#include "handlers.h"
#include "parser.h"
#include "common.h"
#include "log.h"

#ifdef PCAP
#define DEFAULT_SNAPLEN	1500

struct dl_handler {
 pcap_handler f;
 int type;
};

static struct dl_handler dltype[] = {
 { ether_hand,		DLT_EN10MB },  /* ethernet */
 { ether_hand,		DLT_IEEE802 }, /* ethernet */
 { null_hand,		DLT_NULL },    /* loopback */
 { raw_hand,		DLT_RAW },     /* ppp      */
 { NULL,                 0 }, /* aaa... botswanian currency */
};

static pcap_handler lookup_dl_handler(int type) {
 struct dl_handler *p;

 for (p = dltype; p->f; ++p)
  if (type == p->type) {
   return p->f;
  }

 fprintf(stderr, "unknown data link type 0x%x\n", type);
 _exit(1);
}
#endif

/*
void reconfig() {
 logmsg("Reloading ruleset...");
 p_init("NETGUARD_RULES");
} */

void killkenny() {
 logmsg("They killed Kenny! (by signal)");
 #ifdef NGLOG_FILE
 fclose(logf);
 #endif
 _exit(0);
}

void givehelp() {
 #ifdef PCAP
 fprintf(stderr, "usage: netguard [<interface name>]\n");
 #endif
 #ifdef ETHERTAP
 fprintf(stderr, "usage: netguard /dev/tapX\n");
 #endif
 fflush(stderr);
 _exit(0);
}

int main(int argc, char *argv[]) {
#ifdef PCAP
 char ebuf[PCAP_ERRBUF_SIZE];
 static pcap_t *ld;
 int snaplen=DEFAULT_SNAPLEN;
 pcap_handler dl_handler;
 u_char *pcap_userdata;
#endif

// debug=5;
// debug=2;

 if(geteuid()) {
  fprintf(stderr, "Please run as root.\n"); fflush(stderr);
  return 1;
 }

 if(argc>2) givehelp();
 if((argc==2)&&(!strcmp(argv[1], "-h"))) givehelp();
#ifdef ETHERTAP
 if(argc!=2) givehelp();
#endif

/*  signal(SIGHUP, (void *)reconfig); */
 signal(SIGTERM, (void *)killkenny);
 signal(SIGINT, (void *)killkenny);
 signal(SIGUSR1, SIG_IGN);
 signal(SIGUSR2, SIG_IGN);

 iface=argv[1];

#ifdef PCAP
 if(argc==2) iface=argv[1]; else {
  iface=pcap_lookupdev(ebuf);
  if(!iface) {
   fprintf(stderr, "no device was found suitable for packet capture\n");
   return 1;
  }
 }
 
 ld=pcap_open_live(iface, snaplen, 1, 1000, ebuf);
 if(!ld) {
  fprintf(stderr, "pcap_open_live(): %s\n", ebuf);
  return 1;
 }

 dl_handler = lookup_dl_handler(pcap_datalink(ld)); 
 pcap_userdata=0;
#endif

 log_init();
 p_init(NETGUARD_RULES);

 logmsg("NETguard %s is now listening on %s (%d rules)...", NG_VERSION, iface, ruleno);
 fprintf(stdout, "listening on %s\n", iface); fflush(stdout);


#ifndef NGLOG_STDOUT
 if(!debug) {
  if(fork()) _exit(0);
  setsid();
 }
#endif

#ifdef PCAP
 /* Drop privs. */
 seteuid(SAFE_UID); setegid(SAFE_GID);
 setuid(SAFE_UID); setgid(SAFE_GID);
 
 if(pcap_loop(ld, -1, dl_handler, pcap_userdata)<0) {
  logmsg("pcap_loop(): %s\n", pcap_geterr(ld));
  return 1;
 }

 pcap_close(ld);
#endif

#ifdef ETHERTAP
 usetap(iface);
#endif 

 return 0; 
}
