/* netguard-2.0.0/parser.c */
/* Copyright (c) 2000 monk <lizard8@iname.com>. */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "parser.h"
#include "common.h"

FILE *rf;
int ln_no=0;
char *cfln;
struct protoent *myprt;

struct t_cfgpkt *read_rule() { 
 struct t_cfgpkt *myth;
 int msg_f=0, proto_f=0, iface_f=1;

 myth=(struct t_cfgpkt *)malloc(sizeof(struct t_cfgpkt));
 if(!myth) {
  perror("malloc()"); return NULL;
 } 

 fscanf(rf, "%[^\n]\n", cfln);
 while((!feof(rf))&&(memcmp(cfln, "rule begin", 10)))
  fscanf(rf, "%[^\n]\n", cfln);

 if(feof(rf)) return NULL;
 
 myth=(struct t_cfgpkt *)malloc(sizeof(struct t_cfgpkt));
 if(!myth) {
  perror("malloc()"); return NULL;
 }

 myth->tos=-1;
 myth->id=-1;
 myth->ttl=-1;
 myth->proto=0;
 myth->saddr=myth->daddr=-1;
 myth->sport=myth->dport=-1;
 myth->seq=myth->ack=-1;
 myth->f_fin=myth->f_syn=myth->f_rst=myth->f_urg=myth->f_push=myth->f_ack=-1;
 myth->win=-1;
 myth->i_type=myth->i_code=-1;

 while((!feof(rf))&&(memcmp(cfln, "rule end", 8))) {
   if(!memcmp(cfln, "msg ", 4)) {
   if(msg_f)
    logmsg("%d: previous 'msg ' encountered. using current one.\n", ln_no);
   strcpy(myth->msg, cfln+4); msg_f++;
  }

  if(!memcmp(cfln, "proto ", 6)) {
   if(proto_f)
    logmsg("%d: previous 'proto ' encountered. using current one.\n", ln_no);
   myprt=getprotobyname(cfln+6);
   myth->proto=myprt->p_proto;
  }

  if(!memcmp(cfln, "ttl ", 4)) myth->ttl=atoi(cfln+4);
  if(!memcmp(cfln, "id ", 3)) myth->id=atol(cfln+3);
  if(!memcmp(cfln, "tos ", 4)) myth->tos=atoi(cfln+4);
  if(!memcmp(cfln, "iface ", 6)) {
   if(!memcmp(cfln+6, iface, strlen(iface))) iface_f=2; else
    if(iface_f!=2) iface_f=0;
  }
  if(!memcmp(cfln, "saddr ", 6)) {
   myth->saddr=inet_addr(cfln+6);
   if(!(myth->saddr)) {
    logmsg("parser: %s is not a valid IP address\n", cfln+6);
    _exit(1);
   }
  }
  if(!memcmp(cfln, "daddr ", 6)) {
   myth->daddr=inet_addr(cfln+6);
   if(!(myth->daddr)) {
    logmsg("parser: %s is not a valid IP address\n", cfln+6);
    _exit(1);
   }
  }
  if(!memcmp(cfln, "sport ", 6)) myth->sport=htons(atoi(cfln+6));
  if(!memcmp(cfln, "dport ", 6)) myth->dport=htons(atoi(cfln+6));
  if(!memcmp(cfln, "seq ", 4)) myth->seq=ntohl(atol(cfln+4));
  if(!memcmp(cfln, "ack ", 4)) myth->ack=ntohl(atol(cfln+4));
  if(!memcmp(cfln, "winsize ", 8)) myth->win=ntohs(atol(cfln+8));
  if(!memcmp(cfln, "i_type ", 7)) myth->i_type=atoi(cfln+7);
  if(!memcmp(cfln, "i_code ", 7)) myth->i_code=atoi(cfln+7);

  if(!memcmp(cfln, "f_syn 1", 7)) myth->f_syn=1;
  if(!memcmp(cfln, "f_syn 0", 7)) myth->f_syn=0;

  if(!memcmp(cfln, "f_ack 1", 7)) myth->f_ack=1;
  if(!memcmp(cfln, "f_ack 0", 7)) myth->f_ack=0;

  if(!memcmp(cfln, "f_urg 1", 7)) myth->f_urg=1;
  if(!memcmp(cfln, "f_urg 0", 7)) myth->f_urg=0;

  if(!memcmp(cfln, "f_push 1", 8)) myth->f_push=1;
  if(!memcmp(cfln, "f_push 0", 8)) myth->f_push=0;

  if(!memcmp(cfln, "f_rst 1", 7)) myth->f_rst=1;
  if(!memcmp(cfln, "f_rst 0", 7)) myth->f_rst=0;

  fscanf(rf, "%[^\n]\n", cfln);
 }

 if(debug>4) fprintf(stderr, "0x%x\n", (unsigned int)myth); fflush(stderr);
  
 if(!msg_f) {
  logmsg("No MSG field !!!\n"); fflush(stderr);
  myth=NULL;
 }

 if(!iface_f) return read_rule();

 return myth;
}

int p_init(char *fnm) {
 int f=1;
 cfln=(char *)malloc(256);
 if(!cfln) {
  perror("malloc()"); return 0;
 }

 ruleno=0;

 myprt=(struct protoent *)malloc(sizeof(struct protoent));
 if(!myprt) {
  perror("malloc()"); return 0;
 }

 rf=fopen(fnm, "rt");
 if(!rf) rf=fopen("/etc/netguard.rules", "rt");
 if(!rf) {
  perror("fopen()"); return 0;
 }

 first=read_rule(); ruleno++;
 if(!first) {
  logmsg("NO RULES !!!\n");
  return 0;
 }

 first->next=current=read_rule(); ruleno++;

 while(current) {
 // fprintf(stderr, "----- 0x%x - %d\n", (int)current, f); fflush(stderr);
  if(f) {
   first->next=current; f=0;
  }
  current->next=read_rule(); ruleno++;
  current=current->next;
 }
// fprintf(stderr, "\n"); fflush(stderr);

 fclose(rf);

 ruleno--;
 return 1;
}

