/* netguard-2.0.0/tcpip.c */
/* Copyright (c) 2000 monk <lizard8@iname.com>. */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>
#include <string.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>
#include <netinet/igmp.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <pcap.h>
#include "parser.h"
#include "common.h"
#include "log.h"

#define UNDEF	"<undefined>"

static struct protoent *pte;
unsigned int leng, len, myf;
static char *srcad, *dstad;
static char *srcport, *dstport;
static char *prno, *prname;
struct tcphdr *th;
struct udphdr *uh;
int flg;

void check_tcp(struct tcphdr *, struct t_cfgpkt *);
void check_udp(struct udphdr *, struct t_cfgpkt *);
void check_icmp(struct icmphdr *, struct t_cfgpkt *);
void check_igmp(struct igmphdr *, struct t_cfgpkt *);
void check_ip(struct ip *, struct t_cfgpkt *);

char *parsemacros(char *txt) {
 static char *rez;
 int i=0, i1=0, sl=strlen(txt);

 rez=malloc(1024);
 if(!rez) {
  logmsg("[E]: malloc(): cannot allocate memory"); return NULL;
 }
 bzero(rez, 1023);

 while(i<=sl) {
  if(*(txt+i)!='$') {
   memcpy(rez+i1, txt+i, 1);
   i1++;
  }
  else {
   if(!memcmp(txt+i, "$sad", 4)) {
    strcat(rez+i1, srcad); i1+=strlen(srcad); i+=2;
   }
   if(!memcmp(txt+i, "$dad", 4)) {
    strcat(rez+i1, dstad); i1+=strlen(dstad); i+=2;
   }
   if(!memcmp(txt+i, "$spt", 4)) {
    strcat(rez+i1, srcport); i1+=strlen(srcport); i+=2;
   }
   if(!memcmp(txt+i, "$dpt", 4)) {
    strcat(rez+i1, dstport); i1+=strlen(dstport); i+=2;
   }
   if(!memcmp(txt+i, "$pno", 4)) {
    strcat(rez+i1, prno); i1+=strlen(prno); i+=2;
   }
   if(!memcmp(txt+i, "$pro", 4)) {
    strcat(rez+i1, prname); i1+=strlen(prname); i+=2;
   }
   if(!memcmp(txt+i, "$ifc", 4)) {
    strcat(rez+i1, iface); i1+=strlen(iface); i+=2;
   }
   i++;
  }
  i++;
 }
 return rez;
}


void hand_frag(u_char *p, int st, int len) {
 if(debug>2) logmsg("[D]: fragment...\n");
}

void hand_ip(struct pcap_pkthdr *pkt, u_char *p, int st) {
 struct ip *iph;

 iph=(struct ip *)p;
 len=st;

 if((iph->ip_v!=4)&&(debug>2)) {
  logmsg("[E]: %s : IP version %d not supported.", iface, iph->ip_v);
  return;
 }

 if((len<sizeof(struct iphdr))&&(debug)) {
  logmsg("[E]: %s : truncated ip.", iface);
  return;
 }

 leng=ntohs(iph->ip_len);
 if((len<leng)&&(debug))
  logmsg("[E]: %s : truncated ip - %d octets missing.", iface, leng-len);
 len-=iph->ip_hl*4;

 // only process first fragment... for now
 if(ntohs(iph->ip_off)&IP_OFFMASK) hand_frag(p, st, len);

 pte=getprotobynumber(iph->ip_p);
 dstad=inet_ntoa(iph->ip_src);
 srcad=malloc(strlen(dstad)+1);
 strcpy(srcad, dstad);
 dstad=inet_ntoa(iph->ip_dst);

 srcport=malloc(6);
 dstport=malloc(6);
 prno=malloc(6);
 prname=malloc(30);

 switch(iph->ip_p) {
  case IPPROTO_TCP:
   th=(struct tcphdr *)(p+sizeof(struct ip));
   sprintf(srcport, "%d", ntohs(th->source));
   sprintf(dstport, "%d", ntohs(th->dest));
   break;
  case IPPROTO_UDP:
   uh=(struct udphdr *)(p+sizeof(struct ip));
   sprintf(srcport, "%d", ntohs(uh->source));
   sprintf(dstport, "%d", ntohs(uh->dest));
   break;
  default:
   sprintf(srcport, UNDEF);
   sprintf(dstport, UNDEF);
 }

 strcpy(prname, pte->p_name);
 sprintf(prno, "%d", iph->ip_p);

 if(debug>1) {
  logmsg("%s: %s -> %s", iface, srcad, dstad);
  logmsg("%s: protocol=%s, ttl=%d, tos=0x%x", iface, pte->p_name, iph->ip_ttl, iph->ip_tos);
 }

 // check packet...
 current=first;
 while(current) {
  flg=0;
  if((iph->ip_p==current->proto)||(current->proto==IPPROTO_IP)) {
   flg=1;
   check_ip(iph, current);
   switch(iph->ip_p) {
    case IPPROTO_TCP: check_tcp((struct tcphdr *)(p+sizeof(struct ip)), current); break;
    case IPPROTO_UDP: check_udp((struct udphdr *)(p+sizeof(struct ip)), current); break;
    case IPPROTO_ICMP: check_icmp((struct icmphdr *)(p+sizeof(struct ip)), current); break;
    case IPPROTO_IGMP: check_igmp((struct igmphdr *)(p+sizeof(struct ip)), current); break;
   }
   if(flg) logmsg("%s: %s", iface, parsemacros(current->msg));
  }
  current=current->next;
 }
 free(srcad); free(srcport); free(dstport); free(prname); free(prno);
}

void check_ip(struct ip *myh, struct t_cfgpkt *crnt) {
 if((crnt->tos!=-1)&&(crnt->tos!=myh->ip_tos)) flg=0;
 if((crnt->ttl!=-1)&&(crnt->ttl!=myh->ip_ttl)) flg=0;
 if((crnt->id!=-1)&&(crnt->id!=myh->ip_id)) flg=0;
 if((crnt->saddr!=-1)&&(crnt->saddr!=myh->ip_src.s_addr)) flg=0;
 if((crnt->daddr!=-1)&&(crnt->daddr!=myh->ip_dst.s_addr)) flg=0;
}

void check_tcp(struct tcphdr *myh, struct t_cfgpkt *crnt) {
 if((crnt->sport!=-1)&&(crnt->sport!=myh->source)) flg=0;
 if((crnt->dport!=-1)&&(crnt->dport!=myh->dest)) flg=0;
 if((crnt->seq!=-1)&&(crnt->seq!=myh->seq)) flg=0;
 if((crnt->ack!=-1)&&(crnt->ack!=myh->ack_seq)) flg=0;
 if((crnt->win!=-1)&&(crnt->win!=myh->window)) flg=0;
 //logmsg("stored winsize=%d vs. current one=%d", crnt->win, myh->window);
 
 // tcp flags
 if((crnt->f_fin!=-1)&&(crnt->f_fin!=myh->fin)) flg=0;
 if((crnt->f_syn!=-1)&&(crnt->f_syn!=myh->syn)) flg=0;
 if((crnt->f_rst!=-1)&&(crnt->f_rst!=myh->rst)) flg=0;
 if((crnt->f_push!=-1)&&(crnt->f_push!=myh->psh)) flg=0;
 if((crnt->f_ack!=-1)&&(crnt->f_ack!=myh->ack)) flg=0;
 if((crnt->f_urg!=-1)&&(crnt->f_urg!=myh->urg)) flg=0;
}

void check_udp(struct udphdr *myh, struct t_cfgpkt *crnt) {
 if((crnt->sport!=-1)&&(crnt->sport!=myh->source)) flg=0;
 if((crnt->dport!=-1)&&(crnt->dport!=myh->dest)) flg=0;
}

void check_icmp(struct icmphdr *myh, struct t_cfgpkt *crnt) {
 if((crnt->i_type!=-1)&&(crnt->i_type!=myh->type)) flg=0;
 if((crnt->i_code!=-1)&&(crnt->i_code!=myh->code)) flg=0;
}

void check_igmp(struct igmphdr *myh, struct t_cfgpkt *crnt) {
 if((crnt->i_type!=-1)&&(crnt->i_type!=myh->type)) flg=0;
 if((crnt->i_code!=-1)&&(crnt->i_code!=myh->code)) flg=0;
}

